package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.DeactivationApplication;
import digital.laboratory.platform.reagent.service.DeactivationApplicationService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (标准物质到期停用申请表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质到期停用申请表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/deactivation_application" )
@Api(value = "deactivation_application", tags = "(标准物质到期停用申请表)管理")
public class DeactivationApplicationController {

    private final  DeactivationApplicationService deactivationApplicationService;

	/**
	 * 通过id查询(标准物质到期停用申请表)
	 * @param deactivationApplicationId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{deactivationApplicationId}" )
	@PreAuthorize("@pms.hasPermission('reagent_deactivation_application_get')" )
	public R<DeactivationApplication> getById(@PathVariable("deactivationApplicationId" ) String deactivationApplicationId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        DeactivationApplication deactivationApplication = deactivationApplicationService.getById(deactivationApplicationId);
		return R.ok(deactivationApplication);
		//return R.ok(deactivationApplicationService.getById(deactivationApplicationId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param deactivationApplication (标准物质到期停用申请表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_deactivation_application_get')" )
    public R<IPage<DeactivationApplication>> getDeactivationApplicationPage(Page<DeactivationApplication> page, DeactivationApplication deactivationApplication, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<DeactivationApplication> deactivationApplicationSList = deactivationApplicationService.page(page, Wrappers.<DeactivationApplication>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(deactivationApplicationSList);
//		return R.ok(deactivationApplicationService.page(page, Wrappers.query(deactivationApplication)));
    }


    /**
     * 新增(标准物质到期停用申请表)
     * @param deactivationApplication (标准物质到期停用申请表)
     * @return R
     */
    @ApiOperation(value = "新增(标准物质到期停用申请表)", notes = "新增(标准物质到期停用申请表)")
    @SysLog("新增(标准物质到期停用申请表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_deactivation_application_add')" )
    public R<DeactivationApplication> postAddObject(@RequestBody DeactivationApplication deactivationApplication, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        deactivationApplication.setApplicantId(IdWorker.get32UUID().toUpperCase());
		if (deactivationApplicationService.save(deactivationApplication)) {
			return R.ok(deactivationApplication, "对象创建成功");
		}
		else {
			return R.failed(deactivationApplication, "对象创建失败");
		}
    }

    /**
     * 修改(标准物质到期停用申请表)
     * @param deactivationApplication (标准物质到期停用申请表)
     * @return R
     */
    @ApiOperation(value = "修改(标准物质到期停用申请表)", notes = "修改(标准物质到期停用申请表)")
    @SysLog("修改(标准物质到期停用申请表)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_deactivation_application_edit')" )
    public R<DeactivationApplication> putUpdateById(@RequestBody DeactivationApplication deactivationApplication, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (deactivationApplicationService.updateById(deactivationApplication)) {
			return R.ok(deactivationApplication, "保存对象成功");
		}
		else {
			return R.failed(deactivationApplication, "保存对象失败");
		}
    }

    /**
     * 通过id删除(标准物质到期停用申请表)
     * @param deactivationApplicationId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(标准物质到期停用申请表)", notes = "通过id删除(标准物质到期停用申请表)")
    @SysLog("通过id删除(标准物质到期停用申请表)" )
    @DeleteMapping("/{deactivationApplicationId}" )
    @PreAuthorize("@pms.hasPermission('reagent_deactivation_application_del')" )
    public R<DeactivationApplication> deleteById(@PathVariable String deactivationApplicationId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        DeactivationApplication oldDeactivationApplication = deactivationApplicationService.getById(deactivationApplicationId);

		if (deactivationApplicationService.removeById(deactivationApplicationId)) {
			return R.ok(oldDeactivationApplication, "对象删除成功");
		}
		else {
			return R.failed(oldDeactivationApplication, "对象删除失败");
		}

    }

}
